﻿using LibrarianWebApp.Models;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;

namespace LibrarianWebApp.Controllers
{
    public class LibrarianController : Controller
   {
      private static readonly string GetSpaces_Url = "api/librarian/spaces?includeHidden={0}";

      private const string AccessTokenName = "access_token";
      private string _barTenderCloudServiceHostLocation;
      private IConfiguration _configuration;
      private List<Space> _spaces = new();
      private HttpClient _httpClient = new HttpClient();

      public LibrarianController(IConfiguration configuration)
      {
         _configuration = configuration;
         _barTenderCloudServiceHostLocation = _configuration["BarTenderCloudCluster"];
      }

      [Authorize]
      public IActionResult Spaces()
      {
         try
         {
            InitializeHttpClient();

            // Get spaces.
            _spaces = GetSpaces().Result?.Spaces ?? new List<Space>();

            return View("Spaces", _spaces);
         }
         catch (Exception ex)
         {
            TempData["ErrorMessage"] = ex.Message;
            return RedirectToAction("Error");
         }
      }

      public IActionResult Error()
      {
         if (TempData.ContainsKey("ErrorMessage"))
            ViewData["ErrorMessage"] = TempData["ErrorMessage"]?.ToString();
         else
            ViewData["ErrorMessage"] = "";
         return View();
      }

      #region Private Methods
      /// <summary>
      /// Get all spaces.
      /// </summary>
      /// <returns>SpacesCollection</returns>
      private async Task<SpacesCollection> GetSpaces(bool includeHidden = false)
      {
         try
         {
            HttpResponseMessage msg = await _httpClient.GetAsync(string.Format(GetSpaces_Url, includeHidden));

            if (msg.IsSuccessStatusCode)
            {
               string json = await msg.Content.ReadAsStringAsync();
               var stream = new MemoryStream(Encoding.UTF8.GetBytes(json));
               return await JsonSerializer.DeserializeAsync<SpacesCollection>(stream, new JsonSerializerOptions { IncludeFields = true }) ?? new SpacesCollection();
            }

            throw new Exception(msg.ReasonPhrase);
         }
         catch (Exception)
         {
            throw;
         }
      }

      /// <summary>
      /// Initialize http client.
      /// </summary>
      private void InitializeHttpClient()
      {
         string accessToken = HttpContext.GetTokenAsync(AccessTokenName).Result ?? String.Empty;

         _httpClient.DefaultRequestHeaders.Clear();
         _httpClient.BaseAddress = new Uri(_barTenderCloudServiceHostLocation);
         _httpClient.DefaultRequestHeaders.Add("Authorization", $"Bearer {accessToken}");
      }
      #endregion
   }
}
